<?php

namespace Wp618\Com\BaiduAi;

if ( !defined( 'ABSPATH' ) ) exit;

class Model{
    protected $table = ''; // 会被子类覆盖
    protected $is_dev_table = false;
    protected $db;
    protected $app;

    //
    public function __construct()
    {
        global $wpdb;
        $this->db = $wpdb;
        $this->app = App::getInstance();
    }

    //
    public function get_table()
    {
        $prefix = $this->db->prefix;

        if($this->is_dev_table){
            $table = $prefix . $this->app->get_app_item('app_prefix') . $this->table;
        }else{
            $table = $prefix . $this->table;
        }

        return $table;
    }

    //
    public function insert($data, $is_add_time = true){
        if($is_add_time){
            $data['crtime'] = current_time('mysql');
            $data['uptime'] = current_time('mysql');
        }

        if($this->db->insert($this->get_table(), $data)){
            return $this->db->insert_id;
        }

        return 0;
    }

    // format: ['%d', '%f', '%s', '%i']，int, float, string, table_name or field
    // 这里的 where 只允许 等号（=） 值
    public function delete($where, $format = []){
        if(empty($where)){
            return false;
        }

        return $this->db->delete($this->get_table(), $where, $format);
    }

    //
    public function delete_all($check_table){
        $table = $this->get_table();

        if(empty($check_table) && stripos($table, $check_table) !== false){
            return false;
        }

        $sql = 'DELETE FROM ' . $table;

        return $this->db->query($sql);
    }

    //
    public function update($data, $where){
        return $this->db->update($this->get_table(), $data, $where);
    }

    //
    public function get_one($query = []){
        $query['limit'] = 1;

        $result = $this->get_by_query($query);

        return count($result) == 1 ? $result[0] : [];
    }

    //
    public function get_all($query = []){
        return $this->get_by_query($query);
    }

    //
    public function get_count($query = []){
        $sql = [
            'select',
            'count(*)',
            'from',
            $this->get_table(),
        ];

        $q = $this->get_q($sql, $query);

        return (int) $this->db->get_var($q);

    }

    //
    private function get_by_query($query){
        $columns = isset($query['columns']) ? $query['columns'] : '*';
        $sql = [
            'SELECT',
            $columns,
            'FROM',
            $this->get_table(),
        ];

        $q = $this->get_q($sql, $query);

        return $this->db->get_results($q, ARRAY_A);
    }

    //
    private function get_q($sql, $query){
        $values = [];
        // ['name' => 'apple', 'age <=' => 20, 'height %d' => 180]
        if(!empty($query['where'])){
            $sql[] = 'WHERE';
            $sql_where = [];

            foreach ($query['where'] as $key => $val){
                $sql_where[] = $this->get_format_kv($key);

                $values[] = $val;
            }

            $sql[] = implode(' AND ', $sql_where);
        }
        if(isset($query['order'])){
            $query['order'] = trim(preg_replace('#[^\w\s]#', '', $query['order']));

            if(!empty($query['order'])){
                $sql[] = 'ORDER BY ' . $query['order'];
            }
        }
        if(!empty($query['limit'])){
            $sql[] = 'LIMIT ' . intval($query['limit']);
        }
        if(!empty($query['offset'])){
            $sql[] = 'OFFSET ' . intval($query['offset']);
        }

        $sql_str = implode(' ', $sql);
        if(!empty($values)){
            array_unshift($values, $sql_str);

            $q = call_user_func_array([$this->db, 'prepare'], $values);
        }else{
            $q = $sql_str;
        }

        return $q;
    }

    // format: ['%d', '%f', '%s', '%i']，int, float, string, table_name or field
    private function get_format_kv($key){
        $format = '%s';

        $key = trim($key);
        $right_2char = substr($key, -2);
        if(in_array($right_2char, ['%d', '%f', '%s', '%i'], true)){
            $format = $right_2char;
            $key = trim(substr($key, 0, strlen($key) - 2));
        }

        $symbol = '=';
        $right_2char = substr($key, -2);
        if(in_array($right_2char, ['>=', '<=', '<>'], true)){
            $symbol = $right_2char;
            $key = trim(substr($key, 0, strlen($key) - 2));
        }else{
            $right_char = substr($key, -1);
            if(in_array($right_char, ['>', '<'], true)){
                $symbol = $right_char;
                $key = trim(substr($key, 0, strlen($key) - 1));
            }
        }

        return '`' . trim($key) . '` ' . $symbol . ' ' . $format;
    }

    //
    public function last_query(){
        return $this->db->last_query;
    }

    /**
     * Load WordPress dbDelta Function
     */
    public function load_dbDelta()
    {
        if (!\function_exists('dbDelta')) {
            require(ABSPATH . 'wp-admin/includes/upgrade.php');
        }
    }

    //
    public function get_db(){
        return $this->db;
    }

    //
    public function get_prefix(){
        return $this->db->prefix;
    }

}
