<?php

namespace Wp618\Com\BaiduAi;

if ( !defined( 'ABSPATH' ) ) exit;

// 过滤器
class Filter
{
	// 确认本字段需要安全处理
	const SAFE_CONFIRM = 'safe_confirm';

    // 不能传入 array，若传入直接返回 null
    // filters 用 / 分开多个
    public function sanitize($value, $filters = null){
	    if(is_array($value)){
	        return null;
        }

        if(is_null($filters)){
            trigger_error('empty sanitize name', E_USER_ERROR);
        }
        if($filters === self::SAFE_CONFIRM){
            return $value;
        }
	    if(is_string($filters)){
	        // 支持多个过滤器
            $filters = explode('/', $filters);
        }
	    if(!is_array($filters)){
	        return null;
        }

	    // 最后处理，保证能执行到
	    $is_trim = in_array('trim', $filters);
        foreach ($filters as $func){
            if(!is_string($func)){
                trigger_error('sanitize function is not a string', E_USER_ERROR);
            }

            if($func == 'trim'){
                continue;
            }

            // 不正确的过滤方法
            if(substr($func, 0, 1) == '_' || $func == __FUNCTION__ || !method_exists($this, $func)){
                trigger_error('sanitize function error', E_USER_ERROR);
            }

            $value = $this->$func($value);
        }

        if(is_string($value) && $is_trim){
            $value = trim($value);
        }

	    return $value;
    }

    //
    private function bool($val){
	    return boolval($val);
    }

    //
    private function int($val){
	    return intval($val);
    }

    //
    private function absint($val){
	    return abs(intval($val));
    }

    //
    private function posint($val){
	    return max(1, abs(intval($val)));
    }

    //
    private function boolint($val){
	    return empty($val) ? 0 : 1;
    }

    //
    private function num($val){
	    return preg_replace('#\D#', '', $val);
    }

    //
    private function alpha($val){
	    return preg_replace('#[^a-zA-Z]#', '', $val);
    }

    //
    private function alphanum($val){
	    return preg_replace('#[^a-zA-Z0-9]#', '', $val);
    }

    //
    private function word($val){
	    return preg_replace('#[^\w]#', '', $val);
    }

    // \w + 中划线
    private function wdash($val){
	    return preg_replace('#[^\w\-]#', '', $val);
    }

    // \w + 斜杆
    private function wslash($val){
	    return preg_replace('#[^\w/]#', '', $val);
    }

    //
    private function striptags($val){
	    return strip_tags($val);
    }

    //
    private function hex($val){
	    return preg_replace('#[^a-fA-F0-9]#', '', $val);
    }

}
