<?php

namespace Wp618\Com\BaiduAi;

if ( !defined( 'ABSPATH' ) ) exit;

//
class Comments_model extends Model
{
    protected $table = 'comments';
    protected $is_dev_table = false;

    const MAX_REPLY_COMMENT_COUNT = 1000; // 显示的最大子评论数量
    const REPLY_SYMBOL = '{ reply }'; // 显示的最大子评论数量
    const COMMENT_ITEM_TEMPLATE_PATH = __DIR__ . '/../../components/comment-item.php';
    const COMMENT_REPLY_TEMPLATE_PATH = __DIR__ . '/../../components/comment-reply.php';
    const COMMENT_LOAD_TEMPLATE_PATH = __DIR__ . '/../../components/comment-load.php';

    private $item_data = [];
    private $reply_html = '';
    private $load_html = '';

    //
    public function get_post_comments($post_id, $comment_id){
        $limit = get_option('comments_per_page') ? : 10;

        $q = [
            'where' => [
                'comment_post_ID' => $post_id,
                'comment_approved' => '1',
                'comment_parent' => 0,
            ],
            'order' => 'comment_ID desc',
            'limit' => $limit + 1,
        ];

        if($comment_id > 0){
            $q['where']['comment_ID <'] = $comment_id;
        }

        $list = $this->get_all($q);

        $next_id = '';
        if(count($list) > $limit){
            array_pop($list);
            $next_id = $list[$limit - 1]['comment_ID'];
        }

        return [
            'html' => $this->get_list_comment_html($list, 0),
            'next_id' => $next_id,
        ];
    }

    //
    public function load_comment($post_id, $comment_id){
        $res = $this->get_post_comments($post_id, $comment_id);
        $html = $res['html'];
        if(!empty($res['next_id'])){
            $html .= "\n" . $this->get_load_template($res['next_id']);
        }

        return $html;
    }

    //
    private function get_list_comment_html($list, $level){
        $comment_html = '';

        foreach ($list as $comment){
            $comment_html .= $this->get_comment_item($comment, $level) . "\n";
        }

        return $comment_html;
    }

    //
    private function get_comment_item($comment, $level){
        $template = $this->get_item_template();

        $html = $template['html'];
        $matches = $template['matches'];

        $sub_data = $this->get_comment_field_value($comment, $level + 1);

        foreach ($matches[0] as $key => $val){
            $name = $matches[1][$key];
            $replace = isset($sub_data[$name]) ? $sub_data[$name] : '';

            $html = str_ireplace($val, $replace, $html);
        }

        return $html;
    }

    //
    private function get_comment_field_value($comment, $level){
        $data = [
            'avatar' => get_style_js_uri('assets/images/avatar.png'),
            'alt' => esc_attr($comment['comment_author']),
            'author' => esc_html($comment['comment_author']),
            'time' => date('Y-m-d H:i', strtotime($comment['comment_date'])),
            'content' => esc_html($comment['comment_content']),
            'reply' => $this->get_reply_comment($comment, $level),
        ];

        if($comment['user_id'] > 0){
            $user = $this->app->users_model->get_one([
                'where' => ['ID' => $comment['user_id']],
            ]);
            if($user){
                $data['avatar'] = get_avatar_url($user['user_email'], ['size' => 64]);
                $data['alt'] = esc_attr($user['user_nicename']);
                $data['author'] = esc_html($user['user_nicename']);
            }
        }

        return $data;
    }

    //
    private function get_reply_comment($comment, $level){
        $q = [
            'where' => [
                'comment_post_ID' => $comment['comment_post_ID'],
                'comment_approved' => '1',
                'comment_parent' => $comment['comment_ID'],
            ],
            'order' => 'comment_ID desc',
            'limit' => self::MAX_REPLY_COMMENT_COUNT,
        ];

        $list = $this->get_all($q);
        if(empty($list)){
            return '';
        }

        $tpl = $this->get_reply_template();
        $comment_html = $this->get_list_comment_html($list, $level);
        $tpl = str_replace(self::REPLY_SYMBOL, $comment_html, $tpl);

        return $tpl;
    }

    //
    private function get_item_template(){
        if(!empty($this->item_data)){
            return $this->item_data;
        }

        $html = read_file(self::COMMENT_ITEM_TEMPLATE_PATH);
        if(strlen($html) < 1){
            return '';
        }

        $html = preg_replace('#<\?php.*?\?>#s', '', $html);
        preg_match_all('#{\s*comment\.(\S+)\s*}#', $html, $matches);

        $this->item_data = [
            'html' => $html,
            'matches' => $matches,
        ];

        return $this->item_data;
    }

    //
    private function get_reply_template(){
        if(strlen($this->reply_html) > 0){
            return $this->reply_html;
        }

        $html = read_file(self::COMMENT_REPLY_TEMPLATE_PATH);
        if(strlen($html) < 1){
            return '';
        }

        preg_match_all('#<\?php.*?\?>#s', $html, $matches);
        $item_name = basename(self::COMMENT_ITEM_TEMPLATE_PATH);

        foreach ($matches[0] as $key => $val){
            $replace = '';
            if(stripos($val, $item_name) !== false){
                $replace = self::REPLY_SYMBOL;
            }

            $html = str_replace($val, $replace, $html);
        }

        $this->reply_html = $html;

        return $this->reply_html;
    }

    //
    private function get_load_template($next_id){
        if(strlen($this->load_html) > 0){
            return $this->load_html;
        }

        $html = read_file(self::COMMENT_LOAD_TEMPLATE_PATH);
        if(strlen($html) < 1){
            return '';
        }

        $html = preg_replace('#<\?php.*?\?>#s', '', $html);
        $html = preg_replace('#{\s*comment\.next_id\s*}#', $next_id, $html);
        $this->load_html = $html;

        return $this->load_html;
    }

}